@@ -2,18 +2,8 @@ from .component import BipoleComponent
 
 
 class CurrentSource(BipoleComponent):
-    """
-    CurrentSource
 
-    Parameters
-    ----------
-
-    value: Union[str, int, float]
-        The value of the current source.
-    """
-
-    TYPE = 'I'
-    NAME = 'Current Source'
+    type = 'I'
     kinds = {'dc': 'DC', 'ac': 'AC', 'step': 'Step',
              '': 'Arbitrary', 'noise': 'Noise', 's': ''}
     default_kind = ''
@@ -21,4 +11,4 @@ class CurrentSource(BipoleComponent):
     @property
     def sketch_net(self):
 
-        return self.TYPE + ' 1 2 ' + self.kind + ' ' + '; right'
+        return self.type + ' 1 2 ' + self.kind + ' ' + '; right'
