@@ -2,12 +2,9 @@ from .component import BipoleComponent
 
 
 class Diode(BipoleComponent):
-    """
-    Diode
-    """
 
-    TYPE = 'D'
-    NAME = 'Diode'
+    type = 'D'
+    args = ()
     kinds = {'': '', 'led': 'LED', 'photo': 'Photo', 'schottky': 'Schottky',
              'zener': 'Zener', 'zzener': 'Zzener', 'tunnel': 'Tunnel',
              'varcap': 'VarCap', 'bidirectional': 'Bidirectional',
@@ -20,7 +17,7 @@ class Diode(BipoleComponent):
     @property
     def sketch_net(self):
 
-        s = self.TYPE + ' 1 2; right'
+        s = self.type + ' 1 2; right'
         if self.kind != '':
             s += ', kind=' + self.kind
         if self.style != '':
