@@ -2,12 +2,8 @@ from .connection import Connection
 
 
 class Ground(Connection):
-    """
-    Ground connection
-    """
 
-    TYPE = "A"
-    NAME = "Ground"
+    type = "A"
     default_kind = 'ground'
 
     kinds = {'': '', 'ground': 'Ground', 'sground': 'Signal ground',
@@ -16,9 +12,8 @@ class Ground(Connection):
     @property
     def sketch_net(self):
 
-        return self.TYPE + ' 1' '; down, ' + self.kind
+        return self.type + ' 1' '; down, ' + self.kind
 
-    def net(self, connections, step=1):
+    def attr_string(self, step=1):
 
-        return self.name + ' ' + self.nodes[0].name + '; down, ' \
-            + self.kind
+        return 'down, ' + self.kind
