@@ -4,21 +4,12 @@ from numpy.linalg import norm
 
 
 class Opamp(Component):
-    """
-    Opamp
-
-    Parameters
-    ----------
-
-    value: Union[str, int, float]
-        The value of the opamp.
-    """
-
-    TYPE = "E"
-    NAME = "Opamp"
 
+    type = "E"
     sketch_net = 'E 1 2 opamp 3 4'
     sketch_key = 'opamp'
+    label_offset = 0
+    args = ('Ad', 'Ac', 'Ro')
 
     # The Nm node is not used (ground).
     node_pinnames = ('out', '', 'in+', 'in-')
@@ -45,13 +36,6 @@ class Opamp(Component):
              'r-': ('l', -0.85, 0.25),
              'r+': ('l', -0.85, -0.25)}
 
-    def _tf(self, path, scale=1.0):
-
-        # TODO, rotate
-        path = path * self.length() / scale * 2
-        path = path + self.midpoint
-        return path
-
     def assign_positions(self, x1, y1, x2, y2) -> array:
         """Assign node positions based on cursor positions.
 
@@ -72,35 +56,42 @@ class Opamp(Component):
         return positions
 
     @property
-    def midpoint(self) -> float:
+    def midpoint(self):
 
-        pos = (self.nodes[2].position + self.nodes[3].position) / 2
+        pos = (self.nodes[2].pos + self.nodes[3].pos) * 0.5
 
-        return (self.nodes[0].position + pos) / 2
+        return (self.nodes[0].pos + pos) * 0.5
 
     def length(self) -> float:
 
-        pos = (self.nodes[2].position + self.nodes[3].position) / 2
+        pos = (self.nodes[2].pos + self.nodes[3].pos) * 0.5
 
-        diff = (pos - self.nodes[0].position) / 2
-        return norm(diff)
+        diff = (pos - self.nodes[0].pos) * 0.5
+        return diff.norm()
 
-    def net(self, connections, step=1):
-
-        nodenames = [node.name for node in self.nodes]
+    def attr_string(self, step=1):
 
         # TODO: Handle rotation
         dy = abs(self.nodes[3].y - self.nodes[2].y)
         size = dy * 5 / 4
 
-        return '%s %s %s opamp %s %s; right=%s' % (self.name, *nodenames, size)
+        return 'right=%s' % size
 
     def draw(self, editor, layer, **kwargs):
 
-        x1, y1 = self.nodes[2].position
-        x2, y2 = self.nodes[3].position
+        x1, y1 = self.nodes[2].pos.x, self.nodes[2].pos.y
+        x2, y2 = self.nodes[3].pos.x, self.nodes[3].pos.y
 
         xc = (x1 + x2) / 2
         yc = (y1 + y2) / 2
 
-        layer.sketch(self.sketch, offset=(xc, yc), angle=0, lw=2, **kwargs)
+        dy = abs(self.nodes[3].y - self.nodes[2].y)
+        size = dy * 5 / 4
+
+        lw = kwargs.pop('lw', editor.preferences.lw)
+        if self.color != '':
+            kwargs['color'] = self.color
+
+        print(size)
+        layer.sketch(self.sketch, offset=(xc, yc), angle=0, scale=size / 2.5,
+                     lw=lw, **kwargs)
