@@ -2,10 +2,7 @@ from .component import Component
 
 
 class Port(Component):
-    """
-    Port
-    """
 
-    TYPE = "P"
-    NAME = "Port"
+    type = "P"
+    args = ()
     sketch_net = 'P 1 2'
