@@ -2,10 +2,7 @@ from .component import BipoleComponent
 
 
 class Wire(BipoleComponent):
-    """
-    Wire
-    """
 
-    TYPE = 'W'
-    NAME = 'Wire'
+    type = 'W'
+    args = ()
     sketch_net = 'W 1 2'
