@@ -37,7 +37,8 @@ the Heaviside step.
 <h1>Documentation</h1>
 
 For further information about Lcapy, see
- <a href="https://lcapy.readthedocs.io"> https://lcapy.readthedocs.io </a>
+ <a href="https://lcapy-gui.readthedocs.io"> https://lcapy-gui.readthedocs.io </a>
+and  <a href="https://lcapy.readthedocs.io"> https://lcapy.readthedocs.io </a>
 """
 
     def __init__(self):
