@@ -11,8 +11,6 @@ from .drawing import Drawing
 
 class LcapyTk(Tk):
 
-    XSIZE = 36
-    YSIZE = 22
     SCALE = 0.01
 
     GEOMETRY = '1200x800'
@@ -50,6 +48,8 @@ class LcapyTk(Tk):
                                    underline=0, accelerator='Ctrl+n')
         self.file_menu.add_command(label='Open', command=self.on_load,
                                    underline=0, accelerator='Ctrl+o')
+        self.file_menu.add_command(label='Open library', command=self.on_library,
+                                   underline=6, accelerator='Ctrl+l')
         self.file_menu.add_command(label='Save', command=self.on_save,
                                    underline=0, accelerator='Ctrl+s')
         self.file_menu.add_command(label='Save as', command=self.on_save_as,
@@ -224,7 +224,7 @@ class LcapyTk(Tk):
         name = basename(filename)
         self.set_canvas_title(name)
 
-    def create_canvas(self, name):
+    def create_canvas(self, name, model):
 
         tab = Frame(self.notebook)
 
@@ -247,7 +247,7 @@ class LcapyTk(Tk):
         toolbar.update()
         toolbar.pack(side=BOTTOM, fill=X)
 
-        drawing = Drawing(self, fig, self.debug)
+        drawing = Drawing(self, fig, model, self.debug)
         canvas.drawing = drawing
         canvas.tab = tab
         canvas.layer = Layer(canvas.drawing.ax)
@@ -260,10 +260,6 @@ class LcapyTk(Tk):
 
         self.notebook.bind('<<NotebookTabChanged>>', self.on_tab_selected)
 
-        return canvas
-
-    def bind_canvas(self, canvas, model):
-
         canvas.model = model
 
         figure = canvas.drawing.fig
@@ -275,11 +271,12 @@ class LcapyTk(Tk):
 
         self.enter(canvas)
 
+        return canvas
+
     def new(self):
 
         model = self.uimodel_class(self)
-        canvas = self.create_canvas('Untitled')
-        self.bind_canvas(canvas, model)
+        canvas = self.create_canvas('Untitled', model)
         self.model = model
         return model
 
@@ -399,6 +396,11 @@ class LcapyTk(Tk):
 
         self.model.on_laplace_model()
 
+    def on_library(self, *args):
+        from lcapygui import __libdir__
+
+        self.model.on_load(str(__libdir__))
+
     def on_load(self, *args):
 
         self.model.on_load()
@@ -553,11 +555,13 @@ class LcapyTk(Tk):
     def show_info_dialog(self, message):
 
         from tkinter.messagebox import showinfo
+
         showinfo('', message)
 
     def show_error_dialog(self, message):
 
         from tkinter.messagebox import showerror
+
         showerror('', message)
 
     def show_message_dialog(self, message, title=''):
@@ -566,6 +570,12 @@ class LcapyTk(Tk):
 
         self.message_dialog = MessageDialog(message, title)
 
+    def show_warning_dialog(self, message):
+
+        from tkinter.messagebox import showwarning
+
+        showwarning('', message)
+
     def open_file_dialog(self, initialdir='.'):
 
         from tkinter.filedialog import askopenfilename
